//+------------------------------------------------------------------+
//|                                AlexSTAL  .mq4 |
//|                      Copyright  2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#property indicator_chart_window

extern int RefreshPeriodMin = 1;
extern string Instruments1 = "AUDCAD,AUDCHF,AUDDKK,AUDJPY,AUDNOK,AUDNZD,AUDSEK,AUDSGD,AUDUSD,CADCHF,CADJPY,CHFJPY,CHFNOK,CHFSGD,EURAUD,EURCAD,EURCHF,EURDKK,EURGBP,EURHKD,EURJPY,EURNOK,EURNZD,EURSEK,EURSGD,EURUSD,GBPAUD,GBPCAD,GBPCHF,GBPDKK,GBPJPY,GBPNOK,GBPSEK,GBPSGD,GBPUSD,NZDCAD";
extern string Instruments2 = "NZDCHF,NZDJPY,NZDSGD,NZDUSD,SGDJPY,USDCAD,USDCHF,USDDKK,USDHKD,USDJPY,USDMXN,USDNOK,USDSEK,USDSGD,USDZAR,XAGUSD,XAUUSD";
extern string Instruments3 = "";
extern string VerticalLine = "";

string Instruments[];
double Result[][4]; //   ,   -   

extern int StochCount = 7;
extern int Stoch_D = 12;
extern int Stoch_K = 8;
extern int Stoch_S = 5;
extern double Koef = 1.2;

double D[], K[], S[];

//+------------------------------------------------------------------+
datetime LastRefresh;
datetime LastTimeVerticalLine;
//+------------------------------------------------------------------+
void CloseGroupingOfShots(double Massiv[], double& res1, double& res2)
{
	res1 = 0; res2 = 0;
	int i = 0;
	double j = 0;
	double Min = 0, Max = 0, Sred = 0;
	double tmp;
	
	j = ArraySize(Massiv);
	if (j == 0)
		return(0);
	for (i = 0; i < j; i++)
	{
		tmp = Massiv[i];
		Sred += tmp;
		if (i == 0)
		{
			Min = tmp;
			Max = tmp;
		} else {
			Min = MathMin(Min, tmp);
			Max = MathMax(Max, tmp);
		}
	}
	Sred /= j;
	if ( (Sred == 0) || (Max == 0) )
		return(0);
	
	res1 = 100 - (Max - Min);
	res2 = Sred;
}

//+------------------------------------------------------------------+
void GetResultForInstrument(string Instrument, int timeframe, datetime ontime, double& var1, double& var2, double& var3)
{
	int shift = iBarShift(Instrument, timeframe, ontime, false);
	double Value[];
	ArrayResize(Value, StochCount);
	for (int i = 0; i < StochCount; i++)
	 {
		Value[i] = iStochastic(Instrument, timeframe, D[i], K[i], S[i], 3, 1, 0, shift);
		if (i == (StochCount-1))
			var3 = Value[i];
	 }
	double res1 = 0;
	double res2 = 0;
	CloseGroupingOfShots(Value, res1, res2);
	var1 = res1;
	var2 = res2;
}
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
	LastRefresh = TimeCurrent() - RefreshPeriodMin*60;
	//   
	string tmpstr;
	string getinstr;
	int i;
	for (int j = 0; j <3; j++)
	 {
	 	switch(j)
	 	 {
   		case 0:
				tmpstr = Instruments1;
      		break;
   		case 1:
				tmpstr = Instruments2;
      		break;
   		case 2:
				tmpstr = Instruments3;
      		break;
   		default:
				tmpstr = "";
      		break;
	 	 }
		while(true)
	 	{
			i = StringFind(tmpstr, ",", 0);
			if (i != -1)
		 	{
		 		getinstr = StringSubstr(tmpstr, 0, i);
		 		tmpstr = StringSubstr(tmpstr, i+1);
		 	} else {
		 		getinstr = tmpstr;
		 		tmpstr = "";
		 	}
		 	//----------------------------------
		 	getinstr = StringTrimLeft(getinstr);
		 	getinstr = StringTrimRight(getinstr);
		 	if (MarketInfo(getinstr, MODE_BID) != 0)
		  	{
		  		ArrayResize(Instruments, ArraySize(Instruments) + 1);
		  		Instruments[ArraySize(Instruments)-1] = getinstr;
		  	
		  	}
		 	//----------------------------------
		 	if (tmpstr == "")
		 		break;
	 	}
	 }
	//----
	ArrayResize(Result, ArraySize(Instruments));
	
  	//   
	ArrayResize(D, StochCount);
	ArrayResize(K, StochCount);
	ArrayResize(S, StochCount);
	for (i = 0; i < StochCount; i++)
	 {
	 	if (i == 0)
	 	 {
	 	 	D[0] = Stoch_D;
	 	 	K[0] = Stoch_K;
	 	 	S[0] = Stoch_S;
	 	 } else {
	 	 	D[i] = D[i-1] * Koef;
	 	 	K[i] = K[i-1] * Koef;
	 	 	S[i] = S[i-1] * Koef;
	 	 }
	 }
	//----
   if (ObjectType(VerticalLine) != OBJ_VLINE)
   	VerticalLine = "";
	//----
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
	//----
	int j = ArrayRange(Result, 0);
	for (int i = 0; i < j; i++)
	 {
		string objname = "CloseGroupingOfShots" + i;
		if (ObjectFind(objname)>=0)
			ObjectDelete(objname);
	 }
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
	if (ArraySize(Instruments) == 0)
	 {
		Comment("  ");
		return(0);
	 }
	Comment("");
	datetime OnTime;
	if (VerticalLine != "")
	{
		datetime tmptime = ObjectGet(VerticalLine, OBJPROP_TIME1);
		if (LastTimeVerticalLine == tmptime)
			return(0);
		LastTimeVerticalLine = tmptime;
		OnTime = tmptime;
	} else {
		if ( TimeCurrent() < (LastRefresh + RefreshPeriodMin*60) )
			return(0);
		OnTime = TimeCurrent();
	}
	LastRefresh = TimeCurrent();
	
	//----------------------------------------
	int j = ArrayRange(Result, 0);
	int i = j;
	while (i > 0)
	{
    	i--;
    	Result[i][1] = i;
	}
	
	//     
	double var1, var2, var3;
	for (i = 0; i < j; i++)
	 {
	 	GetResultForInstrument(Instruments[i], 0, OnTime, var1, var2, var3);
	 	Result[i][0] = var1;
	 	Result[i][2] = var2;
	 	Result[i][3] = var3;
	 }
	
	//       .
	ArraySort(Result, WHOLE_ARRAY, 0, MODE_DESCEND);
	
	//
	double Res;
	int index;
	string objname;
	for (i = 0; i < j; i++)
	 {
		index = Result[i][1];
		objname = "CloseGroupingOfShots" + i;
		if (ObjectFind(objname)<0)
		 {
			ObjectCreate(objname, OBJ_LABEL, 0, 0, 0);
			ObjectSet(objname, OBJPROP_XDISTANCE, 15);
			ObjectSet(objname, OBJPROP_YDISTANCE, 15 + 10 * i);
		 }
		color ColorLabel;
		Res = Result[i][0];
		if (Res >= 90)
			ColorLabel = Red;
		else
			ColorLabel = DimGray;
		string out = Instruments[index] + " = " + FormatProc(Result[i][0], 4) + "% (" + FormatProc(Result[i][2], 4) + " / ";
		if (Result[i][3] >= 50)
			out = out + FormatProc(Result[i][2] - Result[i][3], 5);
		else
			out = out + FormatProc(Result[i][3] - Result[i][2], 5);
		out = out + ")";
		ObjectSetText(objname, out, 9, "Courier New", ColorLabel);
	 }
   
	//----
   return(0);
}
//+------------------------------------------------------------------+
string FormatProc(double a, int len)
{
	string res = DoubleToStr(a, 1);
	int i = len - StringLen(res);
	for (; i>0; i--)
		res = " " + res;
	return(res);
}