//+------------------------------------------------------------------+
//|                                     Adaptative Stochastic v4.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2005, Luis Guilherme Damiani"
#property link      "http://www.damianifx.com.br"

#property indicator_separate_window
#property indicator_minimum -110
#property indicator_maximum 130
#property indicator_buffers 6
#property indicator_color1 Blue
#property indicator_color2 DarkViolet
#property indicator_color3 DarkViolet
#property indicator_color4 Red
#property indicator_color5 Red
#property indicator_color6 Blue

//---- input parameters
extern int     BasePeriod  = 14; //10
extern int     FastATR     =14;     //7
extern int     SlowATR     =34;    //49
extern bool    Quadratic   =true;
extern int     OverBSLevel =90;
extern int     EntryLevel  =75;
extern int     Maxbars     =900; 
extern int     Step        =2;
extern bool    ScreenAlert = true;
extern bool    PushAlert   = false;

//---- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double ExtMapBuffer4[];
double ExtMapBuffer5[];
double ExtMapBuffer6[];

datetime lastAlert[4];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators   
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexStyle(0,DRAW_LINE,1,2);
   SetIndexBuffer(1,ExtMapBuffer2);
   SetIndexStyle(1,DRAW_LINE,0,1);
   SetIndexBuffer(2,ExtMapBuffer3);
   SetIndexStyle(2,DRAW_LINE,0,1);
   SetIndexBuffer(3,ExtMapBuffer4);
   SetIndexStyle(3,DRAW_LINE,1,1);
   SetIndexBuffer(4,ExtMapBuffer5);
   SetIndexStyle(4,DRAW_LINE,1,1);
   SetIndexBuffer(5,ExtMapBuffer6);
   SetIndexStyle(5,DRAW_LINE,2,1);
   lastAlert[0]=0;
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   double multiplier=0;
   int    counted_bars=IndicatorCounted();
   //---- check for possible errors
   if(counted_bars<0) return(-1);
   //---- last counted bar will be recounted
   int limit=Bars-counted_bars;
   if(limit>Maxbars)limit=Maxbars;
   if (limit>Bars-BasePeriod*2-1)limit=Bars-BasePeriod*2-1;
   double adapt_stoch=0;
   //Comment(Bars+"    "+limit); 
   int periods=0;
   static double factor=0;
   for (int shift = 0; shift<=limit;shift++)
   {  
      if(MathMod(shift,Step)==0)
      {
         //factor=10000*iMACD(NULL,0,adapt_fast,adapt_slow,1,PRICE_CLOSE,MODE_MAIN,shift);
         //factor=MathAbs(iCCI(NULL,0,adapt_n,PRICE_TYPICAL,shift));
         factor=iATR(NULL,0,FastATR,shift)/iATR(NULL,0,SlowATR,shift);
         if (Quadratic)multiplier = factor*factor;else multiplier=factor;        
         periods=MathRound(BasePeriod*multiplier);
      }
                          
      adapt_stoch=2*(iStochastic(NULL,0,periods,1,3,MODE_EMA,1,MODE_MAIN,shift)-50);
      
      ExtMapBuffer1[shift]=adapt_stoch;
      ExtMapBuffer2[shift]=EntryLevel;
      ExtMapBuffer3[shift]=-EntryLevel;
      ExtMapBuffer4[shift]=OverBSLevel;
      ExtMapBuffer5[shift]=-OverBSLevel;
      ExtMapBuffer6[shift]=0;
   }
            
   if(IsCross(ExtMapBuffer1[2], ExtMapBuffer1[1], ExtMapBuffer1[0], EntryLevel, -EntryLevel) && lastAlert[0] != Time[0])
   {
      //Alert(Symbol(),". ExtMapBuffer1[2]: ", ExtMapBuffer1[2], ", ExtMapBuffer1[1]: ", ExtMapBuffer1[1], ", ExtMapBuffer1[0]: ",ExtMapBuffer1[0], ", EntryLevel: ",EntryLevel, ", -EntryLevel: ",-EntryLevel);
      
      if (ExtMapBuffer1[2] < -EntryLevel)
      {
         if (ScreenAlert) Alert(Symbol(), ", TF", Period(), ", Stochy Buy");
         if (PushAlert) SendNotification(Symbol() + ", TF" + Period() + ", Stochy BUY ");
         lastAlert[0]= Time[0]; // need this so only 1 alert happens for a bid
      }
      
      if (ExtMapBuffer1[2] > EntryLevel)
      {
         if (ScreenAlert) Alert(Symbol(), ", TF", Period(), ", Stochy Sell");
         if (PushAlert) SendNotification(Symbol() + ", TF" + Period() + ", Stochy SELL ");
         lastAlert[0]= Time[0]; // need this so only 1 alert happens for a bid
      }
      
   }
   
   return(0);
}
//+------------------------------------------------------------------+

   //These match the "if" IsCross line before the alerts
bool IsCross(double price2BidsAgo, double lastPrice, double currentPrice, int posEntryLevel, int negEntryLevel)
{
   bool rc = false;
   
   if((price2BidsAgo >= posEntryLevel && lastPrice < posEntryLevel && currentPrice < posEntryLevel) ||
      (price2BidsAgo <= negEntryLevel && lastPrice > negEntryLevel && currentPrice > negEntryLevel))
   {
      rc = true;
   }
   
   return(rc);
}