/*------------------------------------------------------------------------------------
   Name: Stoch-MTF.mq4
   Copyright 2012, Xaphod, http://www.xaphod.com
   
   Description: MTF Stochastic with linear interpoation between bars on the higher 
                timeframes. Features:
                - Linear interpolation of values between bars.
                - Dots can be used to tag the closing lower time-frame bar on the line.
                - Dual color to denote rising/over bought and Falling/oversold.

   Change log: 
       2012-07-16. Xaphod, v1.00 
          - First Release 
-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "Copyright 2012, xaphod.com"
#property link      "http://www.xaphod.com"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1 RoyalBlue
#property indicator_width1 2
#property indicator_color2 Red
#property indicator_width2 2
#property indicator_color3 Gold
#property indicator_width3 2

#property indicator_color4 MediumBlue
#property indicator_width4 0
#property indicator_color5 SkyBlue
#property indicator_width5 1
#property indicator_style5 STYLE_DOT
#property indicator_color6 LightSkyBlue
#property indicator_width6 1
#property indicator_style6 STYLE_DOT


#property indicator_level1 20
#property indicator_level2 50
#property indicator_level3 80
#property indicator_levelstyle STYLE_DOT
#property indicator_levelcolor DimGray
#property indicator_minimum 0
#property indicator_maximum 100

//#include <xPrint.mqh>

// Constant definitions
#define INDICATOR_NAME "Stoch"
#define INDICATOR_VERSION "1.00"

// Indicator parameters
extern string    Indi.Version= INDICATOR_VERSION;
extern string    TimeFrame.Settings="";
extern string    TimeFrame.TF="0";          // Timeframe: M1,M5,M15,M30;H1,H4,D1,W1,MN1. OR 1,5,15,30,60,240,1440,10080,43200. Current Timeframe=0.
extern int       TimeFrame.Auto=1;          // Automatically select higher TF. M15 and M30 -> H1. Off=0, 1st HTF=1, 2nd HTF=2
extern string    Stoch.Settings="";
extern int       Stoch.KPeriod=11;          // %K line period
extern int       Stoch.DPeriod=3;           // %D line period
extern int       Stoch.Slowing=3;           // Slowing value
extern int       Stoch.MAMethod=MODE_SMA;   // SMA=0, EMA=1, SMMA=2, LWMA=3
extern int       Stoch.PriceField=1;        // High/Low=0, Close/Close=1
extern int       Stoch.Low=20;              // Low level value
extern int       Stoch.High=80;             // Hig level value
extern string    Show.Settings="";
extern int       Show.MaxBars=0;            // Max bars to draw. Set to 0 to draw full chart history.
extern bool      Show.BarClose=True;        // Draw dot at stochastic value on the closing bar
extern bool      Show.2Color=True;          // Use 2 colors. 1: Rising and/or above high level. 2: Falling and/or below low level
extern bool      Show.DPeriod=True;         // Draw line on stochastic value


// Global module varables
double gadStochKline[];
double gadStochKopen[];
double gadStochKdot[];
double gadStochDline[];
double gadStochDopen[];
double gadStochKDn[];

int giRepaintBars;
int giTimeFrame;
int giMinData;
bool gbDataError=False;


//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {
  int iTF;
  IndicatorDigits(1);
  // Set buffers
  SetIndexStyle(0, DRAW_LINE);
  SetIndexBuffer(0, gadStochKline);
  SetIndexLabel(0,"Stoch K line");
  SetIndexStyle(1, DRAW_LINE);
  SetIndexBuffer(1, gadStochKDn);
  SetIndexLabel(1,"Dn");
  SetIndexStyle(2, DRAW_LINE);
  SetIndexBuffer(2, gadStochKopen);
  SetIndexLabel(2,"Stoch K open");
  SetIndexStyle(3,DRAW_ARROW);
  SetIndexArrow(3,159);
  SetIndexBuffer(3, gadStochKdot);
  SetIndexLabel(3,NULL);

  
  SetIndexStyle(4, DRAW_LINE);
  SetIndexBuffer(4, gadStochDline);
  SetIndexLabel(4,"Stoch D line");
  SetIndexStyle(5, DRAW_LINE);
  SetIndexBuffer(5, gadStochDopen);
  SetIndexLabel(5,NULL);
  
  
  // Set Timeframe
  iTF=Str2TF(TimeFrame.TF);
  switch(TimeFrame.Auto) {
    case 1: 
      giTimeFrame=NextHigherTF(iTF); 
      giRepaintBars=giTimeFrame/Period()+2;     
    break;
    case 2: 
      giTimeFrame=NextHigherTF(NextHigherTF(iTF));
      giRepaintBars=giTimeFrame/Period()+2;
    break;
    default: 
      if (iTF==0 || iTF==Period()) {
        giTimeFrame=Period();
        giRepaintBars=0;
      }
      else {
        giTimeFrame=iTF;
        giRepaintBars=giTimeFrame/Period()+2;
      }
    break;
  }
  
  if (Show.MaxBars<1 || Show.MaxBars>(Bars-Stoch.KPeriod))
    Show.MaxBars=Bars-Stoch.KPeriod;
  Show.MaxBars=MathMin(Show.MaxBars,iBars(NULL,giTimeFrame)*giTimeFrame/Period());
  
  // Name & Stuff
  IndicatorShortName(TF2Str(giTimeFrame)+"-Stoch("+Stoch.KPeriod+","+Stoch.DPeriod+","+Stoch.Slowing+") ");
  
  return(0);
}

//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
   return (0);
}


///-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  int iNewBars;
  int iCountedBars; 
  int i,j,x0,x1;  
  int iHTF, iCTF;
  double y0,y1;
  static int iCnt;
  
  //HiResTimer(0,TIMER_START); 
  // Get unprocessed ticks
  iCountedBars=IndicatorCounted();
  if(iCountedBars < 0) return (-1); 
  if(iCountedBars>0) iCountedBars--;
  
  // Set bars to redraw
  if (NewBars(giTimeFrame)>3)
    iNewBars=Bars-1;
  else
    iNewBars=Bars-iCountedBars;
  iNewBars=MathMin(MathMax(iNewBars,giRepaintBars),Show.MaxBars);
  
  for(iCTF=iNewBars; iCTF>=0; iCTF--) {
    // get higher time-frame bar index for the current bar 
    if (giTimeFrame>Period())
      iHTF=iBarShift(NULL, giTimeFrame, Time[iCTF]);
    else
      iHTF=iCTF;
        
    // On new higher time-frame bar interpolate between the close prices of the previous two bars
    if (Time[iCTF+1]<iTime(NULL, giTimeFrame, iHTF)) {
      // Plot dot on close price
      if (Show.BarClose)
        gadStochKdot[iCTF+1]=iStochastic(NULL,giTimeFrame,Stoch.KPeriod,Stoch.DPeriod,Stoch.Slowing,Stoch.MAMethod,Stoch.PriceField,MODE_MAIN,iHTF+1); 
      else
        gadStochKdot[iCTF+1]=EMPTY_VALUE;
      
      // Get the indexes for the current and previous bar close times
      x1=iCTF+1;
      x0=x1+1;
      while(Time[x0]>=iTime(NULL, giTimeFrame, iHTF+1) && x0<Show.MaxBars)
        x0++;

      //Interpolate stoch K line between the close values
      y0=iStochastic(NULL,giTimeFrame,Stoch.KPeriod,Stoch.DPeriod,Stoch.Slowing,Stoch.MAMethod,Stoch.PriceField,MODE_MAIN,iBarShift(NULL, giTimeFrame, Time[x0]));
      y1=iStochastic(NULL,giTimeFrame,Stoch.KPeriod,Stoch.DPeriod,Stoch.Slowing,Stoch.MAMethod,Stoch.PriceField,MODE_MAIN,iBarShift(NULL, giTimeFrame, Time[x1]));
      for (i=x0; i>=x1; i--) {
        gadStochKline[i]=Interpolate(i,x0,x1,y0,y1);
        gadStochKopen[i]=EMPTY_VALUE;
        if (Show.2Color) {
          if ((gadStochKline[i]<gadStochKline[i+1] && gadStochKline[i]<Stoch.High) || gadStochKline[i]<Stoch.Low) {
            gadStochKDn[i]=gadStochKline[i];
            if (gadStochKDn[i+1]==EMPTY_VALUE && gadStochKline[i+1]<Stoch.High)
            gadStochKDn[i+1]=gadStochKline[i+1];
          }
          else {
            gadStochKDn[i]=EMPTY_VALUE;
          }
        }  
      }
      if (Show.DPeriod) {
        //Interpolate stoch D line between the close values
        y0=iStochastic(NULL,giTimeFrame,Stoch.KPeriod,Stoch.DPeriod,Stoch.Slowing,Stoch.MAMethod,Stoch.PriceField,MODE_SIGNAL,iBarShift(NULL, giTimeFrame, Time[x0]));
        y1=iStochastic(NULL,giTimeFrame,Stoch.KPeriod,Stoch.DPeriod,Stoch.Slowing,Stoch.MAMethod,Stoch.PriceField,MODE_SIGNAL,iBarShift(NULL, giTimeFrame, Time[x1]));
        for (i=x0; i>=x1; i--) {
          gadStochDline[i]=Interpolate(i,x0,x1,y0,y1);
          gadStochDopen[i]=EMPTY_VALUE;
        }
      }
    }
    else {
      gadStochKdot[iCTF+1]=EMPTY_VALUE; 
    }
  }  
  
  // Draw open bar stoch
  // Get interval for current bar
  x1=0;
  x0=x1+1;
  while(Time[x0]>=iTime(NULL, giTimeFrame, 0) && x0<Show.MaxBars)
    x0++;
  // Calc stoch for the close prices
  y0=iStochastic(NULL,giTimeFrame,Stoch.KPeriod,Stoch.DPeriod,Stoch.Slowing,Stoch.MAMethod,Stoch.PriceField,MODE_MAIN,1);
  y1=iStochastic(NULL,giTimeFrame,Stoch.KPeriod,Stoch.DPeriod,Stoch.Slowing,Stoch.MAMethod,Stoch.PriceField,MODE_MAIN,0);
  // Interpolate between the close prices
  for (i=x0; i>=x1; i--) {
    gadStochKopen[i]=Interpolate(i,x0,x1,y0,y1);
    if (i!=x0) {
      gadStochKline[i]=EMPTY_VALUE;
      gadStochKDn[i]=EMPTY_VALUE;
    }
  }
  if (Show.DPeriod) {
    y0=iStochastic(NULL,giTimeFrame,Stoch.KPeriod,Stoch.DPeriod,Stoch.Slowing,Stoch.MAMethod,Stoch.PriceField,MODE_SIGNAL,1);
    y1=iStochastic(NULL,giTimeFrame,Stoch.KPeriod,Stoch.DPeriod,Stoch.Slowing,Stoch.MAMethod,Stoch.PriceField,MODE_SIGNAL,0);
    for (i=x0; i>=x1; i--) {
      gadStochDopen[i]=Interpolate(i,x0,x1,y0,y1);
      if (i!=x0) {
        gadStochDline[i]=EMPTY_VALUE;
      }  
    }
  } 
  
  return(0);
}
//+------------------------------------------------------------------+

//-----------------------------------------------------------------------------
// function: Interpolate()
// Description: Linear interpolation between two points
//-----------------------------------------------------------------------------
double Interpolate(int x, int x0, int x1, double y0, double y1 ){
  double k;
  
  // Invalid interval  
  if (x0==x1)
    return(EMPTY_VALUE);
  
  // Equation of a straight line  
  // y=kx+m, k=(y1-y0)/(x1-x0)    
  k=(y1-y0)/(x1-x0);
  x=x-x0;
  return(k*x+y0);
}  


//-----------------------------------------------------------------------------
// function: NewBars()
// Description: Return nr of new bars on a TF
//-----------------------------------------------------------------------------
int NewBars(int iPeriod) {
  static int iPrevSize=0;
  int iNewSize;
  int iNewBars;
  datetime tTimeArray[];
  
  ArrayCopySeries(tTimeArray,MODE_TIME,Symbol(),iPeriod);
  iNewSize=ArraySize(tTimeArray);
  iNewBars=iNewSize-iPrevSize;
  iPrevSize=iNewSize;
  return(iNewBars);
}


//-----------------------------------------------------------------------------
// function: TF2Str()
// Description: Convert time-frame to a string
//-----------------------------------------------------------------------------
string TF2Str(int iPeriod) {
  switch(iPeriod) {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    default: return("M"+iPeriod);
  }
}


//-----------------------------------------------------------------------------
// function: NextHigherTF()
// Description: Select the next higher time-frame. 
//              Note: M15 and M30 both select H1 as next higher TF. 
//-----------------------------------------------------------------------------
int NextHigherTF(int iPeriod) {
  if (iPeriod==0) iPeriod=Period();
  switch(iPeriod) {
    case PERIOD_M1: return(PERIOD_M5);
    case PERIOD_M5: return(PERIOD_M15);
    case PERIOD_M15: return(PERIOD_H1);
    case PERIOD_M30: return(PERIOD_H1);
    case PERIOD_H1: return(PERIOD_H4);
    case PERIOD_H4: return(PERIOD_D1);
    case PERIOD_D1: return(PERIOD_W1);
    case PERIOD_W1: return(PERIOD_MN1);
    case PERIOD_MN1: return(PERIOD_MN1);
    default: return(Period());
  }
}


//-----------------------------------------------------------------------------
// function: Str2TF()
// Description: Convert time-frame string to an integer, ie a period value.
//-----------------------------------------------------------------------------
int Str2TF(string sTF) {
  int iTF;
  sTF = StrUpperCase(sTF);
  if (sTF=="M1" || sTF=="1") iTF=PERIOD_M1;
  else if (sTF=="M5" || sTF=="5") iTF=PERIOD_M5;
  else if (sTF=="M15"|| sTF=="15") iTF=PERIOD_M15;
  else if (sTF=="M30"|| sTF=="30") iTF=PERIOD_M30;
  else if (sTF=="H1" || sTF=="60") iTF=PERIOD_H1;
  else if (sTF=="H4" || sTF=="240") iTF=PERIOD_H4;
  else if (sTF=="D1" || sTF=="1440") iTF=PERIOD_D1;
  else if (sTF=="W1" || sTF=="10080") iTF=PERIOD_W1;
  else if (sTF=="MN1" || sTF=="43200") iTF=PERIOD_MN1;
  else if (sTF=="0") iTF=Period();
  else if (sTF=="") iTF=Period();
  else iTF=StrToInteger(sTF);
  if (iTF<=0) iTF=Period();
  return(iTF);
}


//-----------------------------------------------------------------------------
// function: StrUpperCase()
// Description: Convert a string to uppercase
//-----------------------------------------------------------------------------
string StrUpperCase(string sStr) {
  int iPos;
  int iLen;
  string sRet="";
  int iChar;
  
  iLen=StringLen(sStr);
  for (iPos=0;iPos<iLen;iPos++) {
    iChar = StringGetChar(sStr, iPos);
    if (iChar>96 && iChar<123)
      sRet=sRet+CharToStr(iChar-32);
    else
      sRet=sRet+CharToStr(iChar);
  }
  return(sRet);
}



