//+------------------------------------------------------------------+
//|                                        Stochastic Elasticity.mq4 |
//| Calculates the ratio between price move and Stochastic move.		|
//+------------------------------------------------------------------+
#property link "http://www.forexfactory.com/showthread.php?t=22105"
#property indicator_separate_window
#property indicator_buffers	2
#property indicator_color1		LimeGreen
#property indicator_color2		Red
//#property indicator_minimum	0.0

//---- input parameters
extern int		Stoch_K			= 14,
					Stoch_D			= 3,
					Stoch_Slowing	= 3,
					Periods			= 2,		// Number of periods between values to compare
					AveragingPeriod= 14;		// Period for Signal SMA
extern double	PipFactor		= 100.0;	// Pip move is multiplied by this factor
extern bool		Difference		= false,	// Take difference instead of ratio
					TakeAbs			= false,	// Take absolute value of result
					TakeSqrt			= false;	// Take square root of result

//---- indicator buffers
double Buffer[],
		 Signal[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void init()
{
//---- indicator line
	IndicatorShortName("Stochastic Elasticity");
	SetIndexStyle	(0,DRAW_HISTOGRAM);
	SetIndexStyle	(1,DRAW_LINE);
	SetIndexBuffer	(0,Buffer);
	SetIndexBuffer	(1,Signal);
	SetIndexLabel	(0,"Stochastic Elasticity");
	SetIndexLabel	(1,"Stochastic Elasticity Signal");
}

//+------------------------------------------------------------------+
double Stoch(int i = 0)
{
	return(iStochastic(NULL,0,Stoch_K,Stoch_D,Stoch_Slowing,0,0,0,i));
}

//+------------------------------------------------------------------+
double Elasticity(int i = 0)
{
	double a = Close[i],
			 b = Close[i+Periods],
			 pipMove		= PipFactor * (a-b),
			 stochA		= Stoch(i),
			 stochB		= Stoch(i+Periods),
			 stochMove	= stochA-stochB,
			 elasticity	= 0.0;

	if(Difference)
		elasticity = stochMove - pipMove;
	else
	if(pipMove!=0.0)
		elasticity = stochMove / pipMove;

	if(TakeAbs)
		elasticity = MathAbs(elasticity);

	if(TakeSqrt)
		elasticity = MathSqrt(elasticity);

	return(elasticity);
}


//+------------------------------------------------------------------+
int start()
{
	int start = Bars-1-IndicatorCounted();

//---- indicator line
	for(int i = start; i>=0; i--)
	{
		Buffer[i] = Elasticity(i);
	}


//---- signal line
	for(i = start; i>=0; i--)
	{
		Signal[i] = iMAOnArray(Buffer,Bars,AveragingPeriod,0,MODE_SMA,i);
	}

	return(0);	
}

