//+------------------------------------------------------------------+
//|                            Blau - Double smoothed Stochastic.mq4 |
//|                        William Blau's Double Smoothed Stochastic |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  Red
#property indicator_color2  Green
#property indicator_style1  STYLE_DOT
#property indicator_minimum 0
#property indicator_maximum 100

//
//
//
//
//

extern int    StochPeriod  = 20;
extern int    SmoothA      = 20;
extern int    SmoothB      =  5;
extern int    SignalPeriod =  5;
extern int    SignalMAMode = MODE_SMMA;
extern bool   ShowLevels   = true; 
extern double Level1       = 80;
extern double Level2       = 50;
extern double Level3       = 20;
extern color  zoneColor    = C'30,33,36';

//
//
//
//
//

double MainBuffer[];
double SignalBuffer[];
double savg1[];
double savg2[];
double davg1[];
double davg2[];
string ShortName;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0, SignalBuffer);
   SetIndexBuffer(1, MainBuffer);
   SetIndexBuffer(2, savg1); SetIndexEmptyValue(2,0.00);
   SetIndexBuffer(3, savg2); SetIndexEmptyValue(3,0.00);
   SetIndexBuffer(4, davg1); SetIndexEmptyValue(4,0.00);
   SetIndexBuffer(5, davg2); SetIndexEmptyValue(5,0.00);
   
   SmoothA = MathMax(1,SmoothA);
   SmoothB = MathMax(1,SmoothB);
      
      //
      //
      //
      //
      //
      
      ShortName=MakeUniqueName("Blau''s double smoothed stochastic "," ("+StochPeriod+","+SmoothA+","+SmoothB+")");

   IndicatorShortName(ShortName);
   SetIndexLabel(0,"Stochastic");
   SetIndexLabel(1,"Signal");
   return(0);

}
int deinit()
{
   DeleteBounds();
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;


   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
       limit = Bars - counted_bars;

   //
   //
   //
   //
   //

   
   double alphaA = (2.00/(SmoothA+1.00));
   double alphaB = (2.00/(SmoothB+1.00));
   for(i = limit; i >= 0 ; i--)
   {
      double dlow  =  Low[ArrayMinimum(Low , StochPeriod,i)];
      double dhigh = High[ArrayMaximum(High ,StochPeriod,i)];

      //
      //
      //
      //
      //
            
         savg1[i] = savg1[i+1] + alphaA * (Close[i] - dlow - savg1[i+1]); 
         savg2[i] = savg2[i+1] + alphaB * (savg1[i]        - savg2[i+1]); 
         davg1[i] = davg1[i+1] + alphaA * (dhigh - dlow    - davg1[i+1]); 
         davg2[i] = davg2[i+1] + alphaB * (davg1[i]        - davg2[i+1]); 
                if (davg2[i] != 0)
                     MainBuffer[i] = 100*savg2[i]/davg2[i];
                else MainBuffer[i] = 0;               
   }
   for(i=0; i<limit; i++) SignalBuffer[i]=iMAOnArray(MainBuffer,0,SignalPeriod,0,SignalMAMode,i);
   
   //
   //
   //
   //
   //
   
   if (ShowLevels) UpdateBounds();
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string MakeUniqueName(string first, string rest)
{
   string result = first+(MathRand()%1001)+rest;

   while (WindowFind(result)> 0)
          result = first+(MathRand()%1001)+rest;
   return(result);
}

//
//
//
//
//

void DeleteBounds()
{
   ObjectDelete(ShortName+"-1");
   ObjectDelete(ShortName+"-2");
   ObjectDelete(ShortName+"-3");
}
void UpdateBounds()
{
   if (Level1 > 0) SetUpBound(ShortName+"-1",100        ,Level1);
   if (Level2 > 0) SetUpBound(ShortName+"-2",Level2*1.01,Level2*0.99);
   if (Level3 > 0) SetUpBound(ShortName+"-3",Level3     ,       0.00);
}
void SetUpBound(string name, double up, double down)
{
   if (ObjectFind(name) == -1)
      {
         ObjectCreate(name,OBJ_RECTANGLE,WindowFind(ShortName),0,0);
         ObjectSet(name,OBJPROP_PRICE1,up);
         ObjectSet(name,OBJPROP_PRICE2,down);
         ObjectSet(name,OBJPROP_COLOR,zoneColor);
         ObjectSet(name,OBJPROP_BACK,true);
         ObjectSet(name,OBJPROP_TIME1,iTime(NULL,0,Bars-1));
      }
      ObjectSet(name,OBJPROP_TIME2,iTime(NULL,0,0));
}