//+------------------------------------------------------------------+
//|                                                     StochRSI.mq4 |
//|                                                                  |
//| Stochastic RSI                                                   |
//|                                                                  |
//| Algorithm taken from book "The New Technical Trader" by T Chande |
//|                                                                  |
//|                                 original code by Witold Wozniak" |
//|                                          http://www.mqlsoft.com/ |
//|                                                                  |
//|                                        adapted code by Linuxser  |
//|                                       http://www.linuxser.com.ar |
//|                                         http://www.forex-tsd.com |
//|                                                    November 2008 |
//+------------------------------------------------------------------+
#property copyright "Linuxser under GNU V2 license"
#property link      "www.linuxser.com.ar and www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 DeepSkyBlue
#property indicator_color2 PaleVioletRed
#property indicator_color3 DimGray
#property indicator_color4 Red
#property indicator_style1 STYLE_DOT
#property indicator_style2 STYLE_DOT
#property indicator_style3 STYLE_DOT

#property indicator_minimum -2
#property indicator_maximum 102
//
//
//
//
//

#import "dynamicZone.dll"
   double dzBuyP(double& sourceArray[],double probabiltyValue, int lookBack, int bars, int i, double precision);
   double dzSellP(double& sourceArray[],double probabiltyValue, int lookBack, int bars, int i, double precision);
#import

double StocRSI[];
double Value3[];
double bliBuffer[];
double sliBuffer[];
double zliBuffer[];

//---- input parameters
/*************************************************************************
PRICE_CLOSE    0 Close price. 
PRICE_OPEN     1 Open price. 
PRICE_HIGH     2 High price. 
PRICE_LOW      3 Low price. 
PRICE_MEDIAN   4 Median price, (high+low)/2. 
PRICE_TYPICAL  5 Typical price, (high+low+close)/3. 
PRICE_WEIGHTED 6 Weighted close price, (high+low+close+close)/4. 
You must use the numeric value of the Applied Price that you want to use
when you set the 'applied_price' value with the indicator inputs.
**************************************************************************/

extern int    RSILength = 14;
extern int    PriceMode = 0;
extern int    DzLookBackBars         = 35;
extern double DzStartBuyProbability  = 0.25;
extern double DzStartSellProbability = 0.25;
int buffers = 0;
int drawBegin = 0;
string fast_name;

int init() {
    drawBegin = RSILength;
    IndicatorBuffers(5);
    SetIndexBuffer(0,bliBuffer);
    SetIndexBuffer(1,sliBuffer);
    SetIndexBuffer(2,zliBuffer);
    SetIndexBuffer(3,StocRSI);
    SetIndexBuffer(4,Value3);
    
   switch(PriceMode)
   {
   case 1 : fast_name="Open"; break;
   case 2 : fast_name="High"; break;
   case 3 : fast_name="Low"; break;
   case 4 : fast_name="Median"; break;
   case 5 : fast_name="Typical"; break;
   case 6 : fast_name="Weighted"; break;
   default: PriceMode=0; fast_name="Close";
   }
    
    IndicatorShortName("Stochastic RSI [" + RSILength + ", "+fast_name+"]");
    return (0);
}

int start() {
    if (Bars <= drawBegin) return (0);
    int countedBars = IndicatorCounted();
    if (countedBars < 0) return (-1);
    if (countedBars > 0) countedBars--;
    int s, limit = Bars - countedBars - 1;
    for (s = limit; s >= 0; s--) {
        double rsi = iRSI(0, 0, RSILength, PriceMode, s);
        double hh = rsi, ll = rsi;
        for (int i = 0; i < RSILength; i++) {
            double tmp = iRSI(0, 0, RSILength, PriceMode, s + i);
            hh = MathMax(hh, tmp);
            ll = MathMin(ll, tmp);
        }
        double Value1 = rsi - ll;
        double Value2 = hh - ll;
        Value3[s] = 0.0;
        if (Value2 != 0.0) {
            Value3[s] = 100* (Value1 / Value2);
        }
        
    }
    for (s = limit - 1; s >= 0; s--) {
        StocRSI[s] = Value3[s];        
      if (DzStartBuyProbability >0) bliBuffer[s] = dzBuyP (StocRSI, DzStartBuyProbability,  DzLookBackBars, Bars, s, 0.0001);
      if (DzStartSellProbability>0) sliBuffer[s] = dzSellP(StocRSI, DzStartSellProbability, DzLookBackBars, Bars, s, 0.0001);
                                    zliBuffer[s] = dzSellP(StocRSI, 0.5                   , DzLookBackBars, Bars, s, 0.0001);
    }
    return (0);   
}

