//+------------------------------------------------------------------+
//|                                              stochastic momentum |
//|                                                           mladen |
//|                                                                  |
//| Stochastic momentum developed by William Blau                    |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers    4
#property indicator_color1     LimeGreen
#property indicator_color2     Magenta
#property indicator_color3     Red
#property indicator_color4     Goldenrod
#property indicator_minimum    -100
#property indicator_maximum    100
#property indicator_level1     0
#property indicator_level2     50
#property indicator_level3     -50
#property indicator_levelcolor DarkSlateGray


//
//
//
//
//

extern int    Length                    = 14;
extern int    Smooth1                   = 7;
extern int    Smooth2                   = 2;
extern int    Signal                    = 5;
extern int    Price                     = PRICE_CLOSE;
extern int    DivergearrowSize          = 1; 
extern bool   drawDivergences           = true;
extern bool   drawIndicatorTrendLines   = true;
extern bool   drawPriceTrendLines       = true;
extern string drawLinesIdentificator    = "SMIdiverge1";
extern string ___                       = "Divergence Alerts";
extern bool   divergenceAlert           = true;
extern bool   divergenceAlertsMessage   = true;
extern bool   divergenceAlertsSound     = true;
extern bool   divergenceAlertsNotify    = false;
extern bool   divergenceAlertsEmail     = false;
extern string divergenceAlertsSoundName = "alert1.wav";

//
//
//
//
//

double bullishDivergence[];
double bearishDivergence[];
double smo[];
double sms[];
double tBuffer[][6];

string indicatorName;
string labelNames;

double pipMultiplier = 1;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,bullishDivergence); SetIndexStyle(0,DRAW_ARROW,0,DivergearrowSize); SetIndexArrow(0,233);
   SetIndexBuffer(1,bearishDivergence); SetIndexStyle(1,DRAW_ARROW,0,DivergearrowSize); SetIndexArrow(1,234);  
   SetIndexBuffer(2,smo); SetIndexLabel(2,"Stochastic momentum");
   SetIndexBuffer(3,sms); SetIndexLabel(3,"Stochastic momentum signal");

   //
   //
   //
   //
   //

   Length  = MathMax(Length ,1);
   Smooth1 = MathMax(Smooth1,1);
   Smooth2 = MathMax(Smooth2,1);
   
   //
   //
   //
   //
   //
       
   labelNames    = "SMI_DivergenceLine "+drawLinesIdentificator+":";
   indicatorName = " Stochastic momentum ("+Length+","+Smooth1+","+Smooth2+","+Price+")";
IndicatorShortName(indicatorName);
return(0);
}

//
//
//
//
//

int deinit()
{  
   int length=StringLen(labelNames);
   for(int i=ObjectsTotal()-1; i>=0; i--)
   {
      string name = ObjectName(i);
      if(StringSubstr(name,0,length) == labelNames)  ObjectDelete(name);   
   }

return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define ema10 0
#define ema11 1
#define ema12 2
#define ema20 3
#define ema21 4
#define ema22 5

int start()
{
   int counted_bars=IndicatorCounted();
   int i,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (ArrayRange(tBuffer,0) != Bars) ArrayResize(tBuffer,Bars);
         if (Digits==3 || Digits==5) 
               pipMultiplier = 10;
         else  pipMultiplier = 1; 
   
         double alpha1 = 2.0 /(1.0+Smooth1);
         double alpha2 = 2.0 /(1.0+Smooth2);
         double alphas = 2.0 /(1.0+Signal);
         
   //
   //
   //
   //
   //
   //

   for(i=limit,r=Bars-i-1; i >= 0; i--,r++)
   {
      double hh = High[iHighest(NULL,0,MODE_HIGH,Length,i)];
      double ll =  Low[ iLowest(NULL,0,MODE_LOW ,Length,i)];
      double pr = iMA(NULL,0,1,0,MODE_SMA,Price,i);

         tBuffer[r][ema10] = pr - 0.5*(hh+ll);
         tBuffer[r][ema20] =           hh-ll;
         if (i>=Bars-3)
         {
            tBuffer[r][ema11] = tBuffer[r][ema10];
            tBuffer[r][ema12] = tBuffer[r][ema10];
            tBuffer[r][ema21] = tBuffer[r][ema20];
            tBuffer[r][ema22] = tBuffer[r][ema20];
            continue;
         }
      
      //
      //
      //
      //
      //
      
      tBuffer[r][ema11] = tBuffer[r-1][ema11] + alpha1*(tBuffer[r][ema10]-tBuffer[r-1][ema11]);
      tBuffer[r][ema12] = tBuffer[r-1][ema12] + alpha2*(tBuffer[r][ema11]-tBuffer[r-1][ema12]);
      tBuffer[r][ema21] = tBuffer[r-1][ema21] + alpha1*(tBuffer[r][ema20]-tBuffer[r-1][ema21]);
      tBuffer[r][ema22] = tBuffer[r-1][ema22] + alpha2*(tBuffer[r][ema21]-tBuffer[r-1][ema22]);

         smo[i] = 100.00 * tBuffer[r][ema12] / (0.5 * tBuffer[r][ema22]);
         if (Signal>1)
            sms[i] = sms[i+1]+alphas*(smo[i]-sms[i+1]);
            if (drawDivergences)
            {
              CatchBullishDivergence(i);
              CatchBearishDivergence(i);
            }            
   }
   return(0);
}

//
//
//
//
//

void CatchBullishDivergence(int shift)
{
   shift++;
         bullishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(labelNames+"l"+DoubleToStr(Time[shift],0));
            ObjectDelete(labelNames+"l"+"os" + DoubleToStr(Time[shift],0));            
   if(!IsIndicatorLow(shift)) return;  

   //
   //
   //
   //
   //
      
   int currentLow = shift;
   int lastLow    = GetIndicatorLastLow(shift+1);

   //
   //
   //
   //
   //
    
   if(smo[currentLow] > smo[lastLow] && Low[currentLow] < Low[lastLow])
   {
      bullishDivergence[currentLow] = smo[currentLow] - Point*pipMultiplier;
      if(drawPriceTrendLines == true)
                 DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow],Green,STYLE_SOLID);
      if(drawIndicatorTrendLines == true)
                 DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],smo[currentLow],smo[lastLow],Green,STYLE_SOLID);
      if(divergenceAlert == true)
                 DisplayAlert("Classical bullish divergence",currentLow);  
                        
   }
     
   //
   //
   //
   //
   //
        
   if(smo[currentLow] < smo[lastLow] && Low[currentLow] > Low[lastLow])
   {
      bullishDivergence[currentLow] = smo[currentLow] - Point*pipMultiplier;
      if(drawPriceTrendLines == true)
                 DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow], Green, STYLE_DOT);
      if(drawIndicatorTrendLines == true)                            
                 DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],smo[currentLow],smo[lastLow], Green, STYLE_DOT);
      if(divergenceAlert == true)
                 DisplayAlert("Reverse bullish divergence",currentLow); 
   }
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CatchBearishDivergence(int shift)
{
   shift++; 
         bearishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(labelNames+"h"+DoubleToStr(Time[shift],0));
            ObjectDelete(labelNames+"h"+"os" + DoubleToStr(Time[shift],0));            
   if(IsIndicatorPeak(shift) == false) return;
   int currentPeak = shift;
   int lastPeak = GetIndicatorLastPeak(shift+1);

   //
   //
   //
   //
   //
      
   if(smo[currentPeak] < smo[lastPeak] && High[currentPeak]>High[lastPeak])
   {
       bearishDivergence[currentPeak] = smo[currentPeak] + Point*pipMultiplier;
       if(drawPriceTrendLines == true)
               DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],Red,STYLE_SOLID);
       if(drawIndicatorTrendLines == true)
               DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],smo[currentPeak],smo[lastPeak],Red,STYLE_SOLID);
       if(divergenceAlert == true)
               DisplayAlert("Classical bearish divergence",currentPeak); 
                        
   }
   
   //
   //
   //
   //
   //

   if(smo[currentPeak] > smo[lastPeak] && High[currentPeak] < High[lastPeak])
   {
      bearishDivergence[currentPeak] = smo[currentPeak] + Point*pipMultiplier;
      if (drawPriceTrendLines == true)
               DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],Red, STYLE_DOT);
      if (drawIndicatorTrendLines == true)
               DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],smo[currentPeak],smo[lastPeak], Red, STYLE_DOT);
      if(divergenceAlert == true)
               DisplayAlert("Reverse bearish divergence",currentPeak);
                         
   }   
}

//
//
//
//
//

bool IsIndicatorPeak(int shift)
{
   if(smo[shift] >= smo[shift+1] && smo[shift] > smo[shift+2] && smo[shift] > smo[shift-1])
       return(true);
   else 
       return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

bool IsIndicatorLow(int shift)
{
   if(smo[shift] <= smo[shift+1] && smo[shift] < smo[shift+2] && smo[shift] < smo[shift-1])
       return(true);
   else 
       return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GetIndicatorLastPeak(int shift)
{
    for(int i = shift+5; i < Bars; i++)
    {
       if(smo[i] >= smo[i+1] && smo[i] > smo[i+2] && smo[i] >= smo[i-1] && smo[i] > smo[i-2])
         return(i);
    }
return(-1);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GetIndicatorLastLow(int shift)
{
    for (int i = shift+5; i < Bars; i++)
    {
       if (smo[i] <= smo[i+1] && smo[i] < smo[i+2] && smo[i] <= smo[i-1] && smo[i] < smo[i-2])
         return(i);
}
     
return(-1);
}

//
//
//
//
//

void DisplayAlert(string doWhat, int shift)
{
    string dmessage;
    static datetime lastAlertTime;
    if(shift <= 2 && Time[0] != lastAlertTime)
    {
      dmessage =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," SMI ",doWhat);
          if (divergenceAlertsMessage) Alert(dmessage);
          if (divergenceAlertsNotify)  SendNotification(dmessage);
          if (divergenceAlertsEmail)   SendMail(StringConcatenate(Symbol()," SMI "),dmessage);
          if (divergenceAlertsSound)   PlaySound(divergenceAlertsSoundName); 
          lastAlertTime = Time[0];
    }
}


//
//
//
//
//

void DrawPriceTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
    string label = labelNames+first+"os"+DoubleToStr(t1,0);
      ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, 0, t1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}

//
//
//
//
//

void DrawIndicatorTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
    int indicatorWindow = WindowFind(indicatorName);
    if (indicatorWindow < 0) return;
    
    //
    //
    //
    //
    //
    
    string label = labelNames+first+DoubleToStr(t1,0);
      ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, indicatorWindow, t1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}