//------------------------------------------------------------------
#property copyright "mladen"
#property link "www.forex-tsd.com"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  LimeGreen
#property indicator_color2  Orange
#property indicator_color3  Orange
#property indicator_color4  Gold
#property indicator_style4  STYLE_DOT
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_minimum 0
#property indicator_maximum 100
#property indicator_level1  20
#property indicator_level2  80

//
//
//
//
//

extern int            StoPeriod       = 32;
extern int            StoSlowing      = 9;
extern int            StoSignal       = 9;
extern ENUM_MA_METHOD StoSignalMethod = MODE_EMA;
extern double         Filter          = 10;
extern bool           colorChangeOnSlopeChange = true;
extern bool           alertsOn        = false;
extern bool           alertsOnCurrent = true;
extern bool           alertsMessage   = true;
extern bool           alertsPushNotif = false;
extern bool           alertsSound     = false;
extern bool           alertsEmail     = false;
extern bool   ShowLines          = false;
extern string linesIdentifier    = "stofiltered lines1";
extern color  linesUpColor       = DeepSkyBlue;
extern color  linesDnColor       = PaleVioletRed;
extern int    LinesStyle         = STYLE_DOT;
extern int    LinesWidth         = 0;

double stoch[];
double stochda[];
double stochdb[];
double signal[];
double slope[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,stoch);
   SetIndexBuffer(1,stochda);
   SetIndexBuffer(2,stochdb);
   SetIndexBuffer(3,signal);
   SetIndexBuffer(4,slope);
   IndicatorShortName("Filtered Stochastic ("+StoPeriod+","+StoSlowing+","+StoSignal+","+DoubleToStr(Filter,2)+")");
   return(0);
}
int deinit()
{
   deleteLines();
return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if (counted_bars<0) return(-1);
      if (counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //

   if (slope[limit]==-1) CleanPoint(limit,stochda,stochdb);
   for (int i=limit; i>=0; i--)
   {
      double close = iSsm(iMA(NULL,0,1,0,MODE_SMA,PRICE_CLOSE,i),Filter,i,0);
      double high  = iSsm(iMA(NULL,0,1,0,MODE_SMA,PRICE_HIGH ,i),Filter,i,1);
      double low   = iSsm(iMA(NULL,0,1,0,MODE_SMA,PRICE_LOW  ,i),Filter,i,2);
        stoch[i]   = iStoch(close,high,low,StoPeriod,StoSlowing,i,0);
        stochda[i] = EMPTY_VALUE;
        stochdb[i] = EMPTY_VALUE;
   }         
   for (i=limit; i>=0; i--)
   {
      signal[i] = iMAOnArray(stoch,0,StoSignal,0,StoSignalMethod,i);
      slope[i]  = slope[i+1];
        if (colorChangeOnSlopeChange)
        {
            if (stoch[i]>stoch[i+1]) slope[i] =  1;
            if (stoch[i]<stoch[i+1]) slope[i] = -1;
        }
        else
        {
            if (stoch[i]>signal[i]) slope[i] =  1;
            if (stoch[i]<signal[i]) slope[i] = -1;
        }            
        if (slope[i] == -1) PlotPoint(i,stochda,stochdb,stoch);
        
        //
        //
        //
        //
        //
     
        if (ShowLines)
        {
          deleteLine(Time[i]);
          if (slope[i]!= slope[i+1])
          {
             if (slope[i] == 1) drawLine(i,linesUpColor);
             if (slope[i] ==-1) drawLine(i,linesDnColor);
          }
        }
   }      
   return(0);
} 

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      string add = "signal line crossed"; if (colorChangeOnSlopeChange) add = "slope changed";
      if (slope[whichBar] != slope[whichBar+1])
      {
         if (slope[whichBar] ==  1) doAlert(whichBar,add+" up");
         if (slope[whichBar] == -1) doAlert(whichBar,add+" down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" stochastic-filered - "+doWhat;
          if (alertsMessage)   Alert(message);
          if (alertsEmail)     SendMail(StringConcatenate(Symbol()," stochastic-filered "),message);
          if (alertsPushNotif) SendNotification(StringConcatenate(Symbol()," stochastic-filered "+message));
          if (alertsSound)     PlaySound("alert2.wav");
   }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

double workSto[][5];
#define _hi 0
#define _lo 1
#define _re 2
#define _ma 3
#define _mi 4
double iStoch(double priceR, double priceH, double priceL, int period, int slowing, int i, int instanceNo=0)
{
   if (ArrayRange(workSto,0)!=Bars) ArrayResize(workSto,Bars); i = Bars-i-1; instanceNo *= 5;
   
   //
   //
   //
   //
   //
   
   workSto[i][_hi+instanceNo] = priceH;
   workSto[i][_lo+instanceNo] = priceL;
   workSto[i][_re+instanceNo] = priceR;
   workSto[i][_ma+instanceNo] = priceH;
   workSto[i][_mi+instanceNo] = priceL;
      for (int k=1; k<period && (i-k)>=0; k++)
      {
         workSto[i][_mi+instanceNo] = MathMin(workSto[i][_mi+instanceNo],workSto[i-k][instanceNo+_lo]);
         workSto[i][_ma+instanceNo] = MathMax(workSto[i][_ma+instanceNo],workSto[i-k][instanceNo+_hi]);
      }                   
      double sumlow  = 0.0;
      double sumhigh = 0.0;
      for(k=0; k<slowing; k++)
      {
         sumlow  += workSto[i-k][_re+instanceNo]-workSto[i-k][_mi+instanceNo];
         sumhigh += workSto[i-k][_ma+instanceNo]-workSto[i-k][_mi+instanceNo];
      }

   //
   //
   //
   //
   //
   
   if(sumhigh!=0.0) 
         return(100.0*sumlow/sumhigh);
   else  return(0);    
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workSsm[][6];
#define _price  0
#define _ssm    1

double workSsmCoeffs[][4];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define Pi 3.14159265358979323846264338327950288

//
//
//
//
//

double iSsm(double price, double period, int i, int instanceNo=0)
{
   if (period<=1) return(price);   
   if (ArrayRange(workSsm,0) !=Bars)                 ArrayResize(workSsm,Bars);
   if (ArrayRange(workSsmCoeffs,0) < (instanceNo+1)) ArrayResize(workSsmCoeffs,instanceNo+1);
   if (workSsmCoeffs[instanceNo][_period] != period)
   {
      workSsmCoeffs[instanceNo][_period] = period;
      double a1 = MathExp(-1.414*Pi/period);
      double b1 = 2.0*a1*MathCos(1.414*Pi/period);
         workSsmCoeffs[instanceNo][_c2] = b1;
         workSsmCoeffs[instanceNo][_c3] = -a1*a1;
         workSsmCoeffs[instanceNo][_c1] = 1.0 - workSsmCoeffs[instanceNo][_c2] - workSsmCoeffs[instanceNo][_c3];
   }

   //
   //
   //
   //
   //

      i = Bars-i-1;
      int s = instanceNo*2;   
          workSsm[i][s+_price] = price;
          workSsm[i][s+_ssm]   = workSsmCoeffs[instanceNo][_c1]*(workSsm[i][s+_price]+workSsm[i-1][s+_price])/2.0 + 
                                 workSsmCoeffs[instanceNo][_c2]*workSsm[i-1][s+_ssm]                              + 
                                 workSsmCoeffs[instanceNo][_c3]*workSsm[i-2][s+_ssm];
   return(workSsm[i][s+_ssm]);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}
void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
         {
            if (first[i+2] == EMPTY_VALUE) 
                 { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
            else { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
         }
   else  { first[i] = from[i]; second[i] = EMPTY_VALUE; }
}

//
//
//
//
//

void drawLine(int i,color theColor)
{
     string name = linesIdentifier+":"+Time[i];
   
     //
     //
     //
     //
     //
      
     ObjectCreate(name,OBJ_VLINE,0,Time[i],0);
        ObjectSet(name,OBJPROP_COLOR,theColor);
        ObjectSet(name,OBJPROP_STYLE,LinesStyle);
        ObjectSet(name,OBJPROP_WIDTH,LinesWidth);
        ObjectSet(name,OBJPROP_BACK,true);
}

//
//
//
//
//

void deleteLines()
{
   string lookFor       = linesIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   
   //
   //
   //
   //
   //
   
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
      if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteLine(datetime time)
{
    string lookFor = linesIdentifier+":"+time; ObjectDelete(lookFor);
}

