//+------------------------------------------------------------------+
//|                                               Stochastic ROC.mq4 |
//|                               Copyright  2013, Gehtsoft USA LLC |
//|                                            http://fxcodebase.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2013, Gehtsoft USA LLC"
#property link      "http://fxcodebase.com"

#property indicator_separate_window
#property indicator_buffers    3
#property indicator_color1     Green
#property indicator_color2     DodgerBlue
#property indicator_color3     Red
#property indicator_width1     2
 

extern int    Kperiod                   = 8;
extern int    Dperiod                   = 3;
extern int    Slowing                   = 3;
extern int    MA_Method                 = 0;
extern int    IndicatorLineIndex        = 0;
extern int    Price_Field               = 0;
extern int    ROC_Period                = 1;
extern string typ                       = "Types are 0 or 1";
extern int    Type                      = 0;
extern int    arrowSize                 = 1; 
extern bool   drawDivergences           = true;
extern bool   ShowClassicalDivergence   = true;
extern bool   ShowHiddenDivergence      = true;
extern bool   drawIndicatorTrendLines   = true;
extern bool   drawPriceTrendLines       = true;
extern string drawLinesIdentificator    = "storocdiverge1";
extern double arrowsDisplacement        = 2.0;
extern bool   divergenceAlert           = true;
extern bool   divergenceAlertsMessage   = true;
extern bool   divergenceAlertsSound     = true;
extern bool   divergenceAlertsEmail     = false;
extern bool   divergenceAlertsNotify    = false;
extern string divergenceAlertsSoundName = "alert1.wav";

double ROC[];
double bullishDivergence[];
double bearishDivergence[];
string indicatorName;
string labelNames;

 
int init()
{
    SetIndexBuffer(0,ROC);
    SetIndexBuffer(1,bullishDivergence); SetIndexStyle(1,DRAW_ARROW,0,arrowSize); SetIndexArrow(1,233);
    SetIndexBuffer(2,bearishDivergence); SetIndexStyle(2,DRAW_ARROW,0,arrowSize); SetIndexArrow(2,234);  
     
      //
      //
      //
      //
      //
      
      labelNames    = "Stochastic_ROC_DivergenceLine "+drawLinesIdentificator+":";
      indicatorName = "Stochastic_ROC Divergence";
      IndicatorShortName(indicatorName);
 return(0);
}
int deinit()
{
   int length=StringLen(labelNames);
   for(int i=ObjectsTotal()-1; i>=0; i--)
   {
      string name = ObjectName(i);
      if(StringSubstr(name,0,length) == labelNames)  ObjectDelete(name);   
   }
   return(0);
}

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-1,Bars-counted_bars);
   
   //
   //
   //
   //
   //
   
   for (i=limit; i>=0; i--)
   { 
      double Now =iStochastic(NULL,0,Kperiod,Dperiod,Slowing,MA_Method,Price_Field,IndicatorLineIndex,i);
	   double Pre =iStochastic(NULL,0,Kperiod,Dperiod,Slowing,MA_Method,Price_Field,IndicatorLineIndex,i+ROC_Period);
	    
		if (Type!= 0 )
		{
	     if (Pre != 0)  ROC[i]=((Now -Pre) /Pre)*100; 
		}
		else
		{
	     ROC[i]=(Now -Pre); 
	   }
	   if (drawDivergences)
      {
         CatchBullishDivergence(i);
         CatchBearishDivergence(i);
      }           
    } 
 return(0);
}

//
//
//
//
//

void CatchBullishDivergence(int shift)
{
   shift++;
         bullishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(labelNames+"l"+DoubleToStr(Time[shift],0));
            ObjectDelete(labelNames+"l"+"os" + DoubleToStr(Time[shift],0));            
   if(!IsIndicatorLow(shift)) return;  

   //
   //
   //
   //
   //
      
   int currentLow = shift;
   int lastLow    = GetIndicatorLastLow(shift+1);
   if (ROC[currentLow] > ROC[lastLow] && Low[currentLow] < Low[lastLow])
   {
     if (ShowClassicalDivergence)
     {
        bullishDivergence[currentLow] = ROC[currentLow] - arrowsDisplacement;
        if (drawPriceTrendLines)    DrawPriceTrendLine    ("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow],Green,STYLE_SOLID);
        if (drawIndicatorTrendLines)DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],ROC[currentLow],ROC[lastLow],Green,STYLE_SOLID);
        if (divergenceAlert)        DisplayAlert          ("Classical bullish divergence",currentLow);  
     }
                        
   }
     
   //
   //
   //
   //
   //
        
   if (ROC[currentLow] < ROC[lastLow] && Low[currentLow] > Low[lastLow])
   {
     if (ShowHiddenDivergence)
     {
        bullishDivergence[currentLow] = ROC[currentLow] - arrowsDisplacement;
        if (drawPriceTrendLines)     DrawPriceTrendLine    ("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow],Green, STYLE_DOT);
        if (drawIndicatorTrendLines) DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],ROC[currentLow],ROC[lastLow],Green, STYLE_DOT);
        if (divergenceAlert)         DisplayAlert          ("Reverse bullish divergence",currentLow); 
     }
   }
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CatchBearishDivergence(int shift)
{
   shift++; 
         bearishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(labelNames+"h"+DoubleToStr(Time[shift],0));
            ObjectDelete(labelNames+"h"+"os" + DoubleToStr(Time[shift],0));            
   if(IsIndicatorPeak(shift) == false) return;
   int currentPeak = shift;
   int lastPeak = GetIndicatorLastPeak(shift+1);

   //
   //
   //
   //
   //
      
   if (ROC[currentPeak] < ROC[lastPeak] && High[currentPeak]>High[lastPeak])
   {
     if (ShowClassicalDivergence)
     {
        bearishDivergence[currentPeak] = ROC[currentPeak] + arrowsDisplacement;
        if (drawPriceTrendLines)     DrawPriceTrendLine    ("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],Red,STYLE_SOLID);
        if (drawIndicatorTrendLines) DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak], ROC[currentPeak],ROC[lastPeak], Red,STYLE_SOLID);
        if (divergenceAlert)         DisplayAlert          ("Classical bearish divergence",currentPeak);
     } 
                        
   }
   
   //
   //
   //
   //
   //

   if (ROC[currentPeak] > ROC[lastPeak] && High[currentPeak] < High[lastPeak])
   {
     if (ShowHiddenDivergence)
     {
        bearishDivergence[currentPeak] = ROC[currentPeak] + arrowsDisplacement;
        if (drawPriceTrendLines)     DrawPriceTrendLine    ("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],Red, STYLE_DOT);
        if (drawIndicatorTrendLines) DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],ROC[currentPeak], ROC[lastPeak], Red, STYLE_DOT);
        if (divergenceAlert)         DisplayAlert          ("Reverse bearish divergence",currentPeak);
     }
                         
   }   
}

//
//
//
//
//

bool IsIndicatorPeak(int shift)
{
   if(ROC[shift] >= ROC[shift+1] && ROC[shift] > ROC[shift+2] && ROC[shift] > ROC[shift-1])
        return(true);
   else return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

bool IsIndicatorLow(int shift)
{
   if(ROC[shift] <= ROC[shift+1] && ROC[shift] < ROC[shift+2] && ROC[shift] < ROC[shift-1])
        return(true);
   else return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GetIndicatorLastPeak(int shift)
{
    for(int i = shift+5; i < Bars; i++)
    {
       if(ROC[i] >= ROC[i+1] && ROC[i] > ROC[i+2] && ROC[i] >= ROC[i-1] && ROC[i] > ROC[i-2])
         return(i);
    }
return(-1);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GetIndicatorLastLow(int shift)
{
    for (int i = shift+5; i < Bars; i++)
    {
       if (ROC[i] <= ROC[i+1] && ROC[i] < ROC[i+2] && ROC[i] <= ROC[i-1] && ROC[i] < ROC[i-2])
         return(i);
}
     
return(-1);
}

//
//
//
//
//

void DisplayAlert(string doWhat, int shift)
{
    string dmessage;
    static datetime lastAlertTime;
    if(shift <= 2 && Time[0] != lastAlertTime)
    {
      dmessage =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Stochastic ROC ",doWhat);
          if (divergenceAlertsMessage) Alert(dmessage);
          if (divergenceAlertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," Stochastic ROC " +" "+dmessage));
          if (divergenceAlertsEmail)   SendMail(StringConcatenate(Symbol()," Stochastic ROC "),dmessage);
          if (divergenceAlertsSound)   PlaySound(divergenceAlertsSoundName); 
          lastAlertTime = Time[0];
    }
}


//
//
//
//
//

void DrawPriceTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
    string label = labelNames+first+"os"+DoubleToStr(t1,0);
      ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, 0, t1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}

//
//
//
//
//

void DrawIndicatorTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
    int indicatorWindow = WindowFind(indicatorName);
    if (indicatorWindow < 0) return;
    
    //
    //
    //
    //
    //
    
    string label = labelNames+first+DoubleToStr(t1,0);
      ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, indicatorWindow, t1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}

