//+------------------------------------------------------------------+
//|                                               Stochastic RVI.mq4 |
//| Original Code for Tradestation from the book of John F. Ehlers'  |
//| 'Cybernetic Analysis for Stocks and Futures'                     |
//| (Original code at bottom)                                        |
//| Linuxser 2007 for TSD    http://www.forex-tsd.com/               |
//+------------------------------------------------------------------+
#property copyright "Under The GNU General Public License"
#property link      "www.gnu.org"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 Red
#property indicator_color2 Blue
#property  indicator_level1  0
#property  indicator_levelcolor  Silver
#property  indicator_levelstyle 2

//---- input parameters
extern int RVI_Periods=8;
double NumOfBars=0;

//---- buffers
double FisherRVI[];
double TriggerRVI[];
double Buffer0[];
double Buffer1[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- 2 additional buffers are used for counting.
   IndicatorBuffers(4);

   SetIndexBuffer(2,Buffer0);   
   SetIndexBuffer(3,Buffer1);
   
//---- indicator lines
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0, FisherRVI);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1, TriggerRVI);
   
   IndicatorShortName("Stochastic RVI ("+RVI_Periods+")");
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//---- TODO: add your code here
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int    counted_bars=IndicatorCounted();
   double Current_RVI,Lowest_RVI,Highest_RVI,Per3;
   double Per1=RVI_Periods;
   double Per2=RVI_Periods;
//---- TODO: add your code here
   if(NumOfBars==0)NumOfBars=Bars;
   
   for(int i=NumOfBars-MathMax(RVI_Periods,Per1)-1;i>=0;i--){
      Current_RVI=iRVI(NULL,0,RVI_Periods,MODE_MAIN,i);
      Highest_RVI=Current_RVI;
      Lowest_RVI=Current_RVI;
      
      for(int x=0;x<=Per1;x++){
         Lowest_RVI=MathMin(Lowest_RVI,iRVI(NULL,0,RVI_Periods,MODE_MAIN,i+x));
         Highest_RVI=MathMax(Highest_RVI,iRVI(NULL,0,RVI_Periods,MODE_MAIN,i+x));
         }

      Per3=0;
      for(x=0;x<=Per2;x++) Per3=Per3+Buffer1[i+x];

      if(Highest_RVI!=Lowest_RVI)      
      Buffer0[i]=(Current_RVI-Lowest_RVI)/(Highest_RVI -Lowest_RVI);      
      Buffer1[i]= (4*Buffer0[i] + 3*Buffer0[i+1] + 2*Buffer0[i+2] + Buffer0[i+3]) / 10;      
      FisherRVI[i]= 2*(Buffer1[i] - 0.5);
      TriggerRVI[i]= 0.96*FisherRVI[i+1]+0.02;
      }           
      
//----
   return(0);
  }
//+------------------------------------------------------------------+
/*
{****************************************************
Stochastic Relative Vigor Index (RVI)
*****************************************************}
Inputs: Length(8);
Vars: Num(0),
Denom(0),
count(0),
RVI(0),
MaxRVI(0),
MinRVI(0);
Value1 = ((Close - Open) + 2*(Close[1] - Open[1])+2*(Close[2] - Open[2]) + (Close[3] - Open[3]))/6;
Value2 = ((High - Low) + 2*(High[1] - Low[1])+2*(High[2] - Low[2]) + (High[3] - Low[3]))/6;
Num = 0;
Denom = 0;
For count = 0 to Length - 1 begin
Num = Num + Value1[count];
Denom = Denom + Value2[count];
End;

If Denom <> 0 then RVI = Num / Denom;
MaxRVI = Highest(RVI, Length);
MinRVI = Lowest(RVI, Length);
If MaxRVI <> MinRVI then Value3 = (RVI - MinRVI)/ (MaxRVI - MinRVI);
Value4 = (4*Value3 + 3*Value3[1] + 2*Value3[2]+ Value3[3]) / 10;
Value4 = 2*(Value4 - .5);
Plot1(Value4, "RVI");
Plot2(.96*(Value4[1] + .02), "Trigger");
Plot3(0,"Ref");
*/