
#property copyright ""

#property indicator_separate_window
#property indicator_buffers    3
#property indicator_color1     Green 
#property indicator_width1     2  
#property indicator_color2     IndianRed  
#property indicator_width2     2  
#property indicator_color3     DarkGreen 
#property indicator_width3     2
#property indicator_levelcolor MediumOrchid 

//
//
// 
//
//

extern int    KPeriod                 = 14;
extern int    DPeriod                 = 3;
extern int    SlowingPeriod           = 3;
extern int    RSIPeriod               = 14;
extern double Uplevel                 = 15;
extern double Dnlevel                 = -15;

extern string note                    = "turn on Alert = true; turn off = false";
extern bool   alertsOn                = false;
extern bool   alertsOnZeroCross       = true;
extern bool   alertsOnLevelCross      = true;
extern bool   alertsOnCurrent         = true;
extern bool   alertsMessage           = true;
extern bool   alertsSound             = false;
extern bool   alertsNotify            = false;
extern bool   alertsEmail             = false;
extern string soundFile               = "alert2.wav";

extern bool   arrowsVisible           = false;
extern string arrowsIdentifier        = "STORSI arrows1";
extern double arrowsUpperGap          = 1.0;
extern double arrowsLowerGap          = 1.0;

extern bool   arrowsOnZeroCross       = true;
extern color  arrowsZeroCrossUpColor  = LimeGreen;
extern color  arrowsZeroCrossDnColor  = Red;
extern int    arrowsZeroCrossUpCode   = 241;
extern int    arrowsZeroCrossDnCode   = 242;

extern bool   arrowsOnLevelCross      = true;
extern color  arrowsLevelCrossUpColor = LimeGreen;
extern color  arrowsLevelCrossDnColor = Red;
extern int    arrowsLevelCrossUpCode  = 241;
extern int    arrowsLevelCrossDnCode  = 242;

double upB[];
double loB[];
double Stoch1[];
double trend1[];
double trend2[];

//
//
//
//
//

int init() 
{

   IndicatorBuffers(5);
   SetIndexBuffer(0,upB); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,loB); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,Stoch1);
   SetIndexBuffer(3,trend1);
   SetIndexBuffer(4,trend2);
   SetLevelValue(0,0);
   SetLevelValue(1,Uplevel);
   SetLevelValue(2,Dnlevel);

   IndicatorShortName("Stoch" + "(" + KPeriod + "," + DPeriod + "," + SlowingPeriod + ")" + " - RSI" + "(" + RSIPeriod + ") Histogram" );

   return(0);
}

//
//
//
//
//

int deinit()
{
   deleteArrows();

return(0);
}

//
//
//
//
//

int start() 
{
   int counted_bars=IndicatorCounted();
   int shift,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
   
   //
   //
   //
   //
   //
   
   for (shift=limit;shift>=0;shift--)   
   {
      double Stoch  = iStochastic(NULL,0,KPeriod,DPeriod,SlowingPeriod,MODE_SMA,0,MODE_MAIN,shift);      
      double RSI    = iRSI(NULL, 0, RSIPeriod, PRICE_CLOSE, shift);
      Stoch1[shift] = Stoch + RSI - 100; 
      
      //
      //
      //
      //
      //
      
      upB[shift] = EMPTY_VALUE;
      loB[shift] = EMPTY_VALUE;
      trend1[shift] = trend1[shift+1];
      trend2[shift] = trend2[shift+1];     
      if(Stoch1[shift]>0)          trend1[shift] = 1;
      if(Stoch1[shift]<0)          trend1[shift] =-1;
      if (Stoch1[shift] > Uplevel) trend2[shift] = 1;
      if (Stoch1[shift] < Dnlevel) trend2[shift] =-1;
      if(trend1[shift] == 1) upB[shift]=Stoch1[shift];  
      if(trend1[shift] ==-1) loB[shift]=Stoch1[shift];  
      manageArrow(shift);
   }
   manageAlerts();
return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void manageArrow(int i)
{
   if (arrowsVisible)
   {
      deleteArrow(Time[i]);
      string lookFor = arrowsIdentifier+":"+Time[i]; ObjectDelete(lookFor);
      if (arrowsOnZeroCross && trend1[i] != trend1[i+1])
      {
         if (trend1[i] == 1) drawArrow(i,arrowsZeroCrossUpColor,arrowsZeroCrossUpCode,false);
         if (trend1[i] ==-1) drawArrow(i,arrowsZeroCrossDnColor,arrowsZeroCrossDnCode, true);
      } 
      if (arrowsOnLevelCross && trend2[i] != trend2[i+1])
      {
         if (trend2[i] == 1) drawArrow(i,arrowsLevelCrossUpColor,arrowsLevelCrossUpCode,false);
         if (trend2[i] ==-1) drawArrow(i,arrowsLevelCrossDnColor,arrowsLevelCrossDnCode, true);
      }                 
   }         
}       

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      
      //
      //
      //
      //
      //
      
      static datetime time1 = 0;
      static string   mess1 = "";
         if (alertsOnZeroCross && trend1[whichBar] != trend1[whichBar+1])
         {
            if (trend1[whichBar] ==  1) doAlert(time1,mess1,whichBar," crossed zero up");
            if (trend1[whichBar] == -1) doAlert(time1,mess1,whichBar," crossed zero down");
         }            
      static datetime time2 = 0;
      static string   mess2 = "";
         if (alertsOnLevelCross && trend2[whichBar] != trend2[whichBar+1])
         {
            if (trend2[whichBar] ==  1) doAlert(time2,mess2,whichBar," crossed upper level up");
            if (trend2[whichBar] == -1) doAlert(time2,mess2,whichBar," crossed lower level down");
         }            
   }
}   

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, int forBar, string doWhat)
{
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol(),TimeToStr(TimeLocal(),TIME_SECONDS)," StochRSIHistogram ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," StochRSIHistogram "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}